#include "stdafx.h"
#include "TVTest.h"
#include "AppMain.h"
#include "CardReaderDialog.h"
#include "DialogUtil.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// T[rXAPIp
#pragma comment(lib, "advapi32.lib")


// X}[gJ[hT[rXLׂ
enum SCardQueryResult {
	SCARD_QUERY_ENABLED,
	SCARD_QUERY_DISABLED,
	SCARD_QUERY_ERR_SERVICENOTFOUND,
	SCARD_QUERY_ERR_OPENMANAGER,
	SCARD_QUERY_ERR_OPENSERVICE,
	SCARD_QUERY_ERR_QUERYCONFIG
};
static SCardQueryResult IsSmartCardServiceEnabled()
{
	SCardQueryResult Result;

	SC_HANDLE hManager=::OpenSCManager(NULL,NULL,GENERIC_READ);
	if (hManager==NULL)
		return SCARD_QUERY_ERR_OPENMANAGER;

	TCHAR szName[256];
	DWORD Length=lengthof(szName);
	if (!::GetServiceKeyName(hManager,TEXT("Smart Card"),szName,&Length))
		::lstrcpy(szName,TEXT("SCardSvr"));

	SC_HANDLE hService=::OpenService(hManager,szName,SERVICE_QUERY_CONFIG);
	if (hService==NULL) {
		if (::GetLastError()==ERROR_SERVICE_DOES_NOT_EXIST)
			Result=SCARD_QUERY_ERR_SERVICENOTFOUND;
		else
			Result=SCARD_QUERY_ERR_OPENSERVICE;
		::CloseServiceHandle(hManager);
		return Result;
	}

	Result=SCARD_QUERY_ERR_QUERYCONFIG;
	DWORD Size=0;
	::QueryServiceConfig(hService,NULL,0,&Size);
	if (Size>0) {
		BYTE *pBuffer=new BYTE[Size];
		QUERY_SERVICE_CONFIG *pConfig=(QUERY_SERVICE_CONFIG*)pBuffer;

		if (::QueryServiceConfig(hService,pConfig,Size,&Size)) {
			if (pConfig->dwStartType==SERVICE_DISABLED)
				Result=SCARD_QUERY_DISABLED;
			else
				Result=SCARD_QUERY_ENABLED;
		}
		delete [] pBuffer;
	}

	::CloseServiceHandle(hService);
	::CloseServiceHandle(hManager);

	return Result;
}




CCardReaderErrorDialog::CCardReaderErrorDialog()
	: m_ReaderType(CCoreEngine::CARDREADER_SCARD)
{
}


CCardReaderErrorDialog::~CCardReaderErrorDialog()
{
}


bool CCardReaderErrorDialog::Show(HWND hwndOwner)
{
	return ::DialogBoxParam(GetAppClass().GetResourceInstance(),MAKEINTRESOURCE(IDD_CARDREADER),
							hwndOwner,DlgProc,reinterpret_cast<LPARAM>(this))==IDOK;
}


bool CCardReaderErrorDialog::SetMessage(LPCTSTR pszMessage)
{
	return m_Message.Set(pszMessage);
}


CCardReaderErrorDialog *CCardReaderErrorDialog::GetThis(HWND hDlg)
{
	return static_cast<CCardReaderErrorDialog*>(::GetProp(hDlg,TEXT("This")));
}


static bool SearchReaders(HWND hDlg)
{
	CCardReader *pCardReader;
	bool fFound=false;
	int Index;

	pCardReader=CCardReader::CreateCardReader(CCardReader::READER_SCARD);
	if (pCardReader!=NULL) {
		if (pCardReader->NumReaders()>0) {
			for (int i=0;i<pCardReader->NumReaders();i++) {
				LPCTSTR pszReaderName=pCardReader->EnumReader(i);
				if (pszReaderName!=NULL) {
					Index=(int)DlgListBox_AddString(hDlg,IDC_CARDREADER_READERLIST,pszReaderName);
					DlgListBox_SetItemData(hDlg,IDC_CARDREADER_READERLIST,Index,(LPARAM)CCoreEngine::CARDREADER_SCARD);
					fFound=true;
				}
			}
		}
		delete pCardReader;
	}

	pCardReader=CCardReader::CreateCardReader(CCardReader::READER_HDUS);
	if (pCardReader!=NULL) {
		if (pCardReader->Open()) {
			Index=(int)DlgListBox_AddString(hDlg,IDC_CARDREADER_READERLIST,TEXT("HDUSJ[h[_"));
			DlgListBox_SetItemData(hDlg,IDC_CARDREADER_READERLIST,Index,(LPARAM)CCoreEngine::CARDREADER_HDUS);
			pCardReader->Close();
			fFound=true;
		}
		delete pCardReader;
	}

	pCardReader=CCardReader::CreateCardReader(CCardReader::READER_PLEX);
	if (pCardReader!=NULL) {
		if (pCardReader->Open()) {
			Index=(int)DlgListBox_AddString(hDlg,IDC_CARDREADER_READERLIST,TEXT("PLEX TVJ[h[_"));
			DlgListBox_SetItemData(hDlg,IDC_CARDREADER_READERLIST,Index,(LPARAM)CCoreEngine::CARDREADER_PLEX);
			pCardReader->Close();
			fFound=true;
		}
		delete pCardReader;
	}

	return fFound;
}


INT_PTR CALLBACK CCardReaderErrorDialog::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			CCardReaderErrorDialog *pThis=reinterpret_cast<CCardReaderErrorDialog*>(lParam);

			::SetProp(hDlg,TEXT("This"),pThis);
			::SendDlgItemMessage(hDlg,IDC_CARDREADER_ICON,STM_SETICON,
				reinterpret_cast<WPARAM>(::LoadIcon(NULL,IDI_WARNING)),0);
			if (!pThis->m_Message.IsEmpty())
				::SetDlgItemText(hDlg,IDC_CARDREADER_MESSAGE,pThis->m_Message.Get());
			bool fFound=SearchReaders(hDlg);
			EnableDlgItem(hDlg,IDC_CARDREADER_RETRY,fFound);
			::CheckRadioButton(hDlg,IDC_CARDREADER_RETRY,IDC_CARDREADER_NOREADER,
							   fFound && pThis->m_ReaderType!=CCoreEngine::CARDREADER_NONE?
							   IDC_CARDREADER_RETRY:IDC_CARDREADER_NOREADER);
			EnableDlgItem(hDlg,IDC_CARDREADER_READERLIST,
						  fFound && pThis->m_ReaderType!=CCoreEngine::CARDREADER_NONE);
			if (fFound)
				DlgListBox_SetCurSel(hDlg,IDC_CARDREADER_READERLIST,0);
			AdjustDialogPos(::GetParent(hDlg),hDlg);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_CARDREADER_RETRY:
		case IDC_CARDREADER_NOREADER:
			EnableDlgItem(hDlg,IDC_CARDREADER_READERLIST,
						  DlgRadioButton_IsChecked(hDlg,IDC_CARDREADER_RETRY));
			return TRUE;

		case IDC_CARDREADER_SEARCH:
			{
				DlgListBox_Clear(hDlg,IDC_CARDREADER_READERLIST);
				HCURSOR hcurOld=::SetCursor(::LoadCursor(NULL,IDC_WAIT));
				bool fFound=SearchReaders(hDlg);
				::SetCursor(hcurOld);
				EnableDlgItem(hDlg,IDC_CARDREADER_RETRY,fFound);
				::CheckRadioButton(hDlg,IDC_CARDREADER_RETRY,IDC_CARDREADER_NOREADER,
								   fFound?IDC_CARDREADER_RETRY:IDC_CARDREADER_NOREADER);
				EnableDlgItem(hDlg,IDC_CARDREADER_READERLIST,
							  fFound && DlgRadioButton_IsChecked(hDlg,IDC_CARDREADER_RETRY));
				if (fFound) {
					DlgListBox_SetCurSel(hDlg,IDC_CARDREADER_READERLIST,0);
				} else {
					TCHAR szMessage[1024];

					::lstrcpy(szMessage,TEXT("J[h[_t܂B"));
					switch (IsSmartCardServiceEnabled()) {
					case SCARD_QUERY_DISABLED:
						::lstrcat(szMessage,TEXT("\nSmart Card T[rXɂȂĂ܂B"));
						break;
					case SCARD_QUERY_ERR_SERVICENOTFOUND:
						::lstrcat(szMessage,TEXT("\nSmart Card T[rXt܂B"));
						break;
					}
					::MessageBox(hDlg,szMessage,TEXT(""),MB_OK | MB_ICONINFORMATION);
				}
			}
			return TRUE;

		case IDOK:
			{
				CCardReaderErrorDialog *pThis=GetThis(hDlg);

				if (DlgRadioButton_IsChecked(hDlg,IDC_CARDREADER_RETRY)) {
					int Sel=(int)DlgListBox_GetCurSel(hDlg,IDC_CARDREADER_READERLIST);

					if (Sel<0) {
						::MessageBox(hDlg,TEXT("J[h[_IĂB"),TEXT("肢"),MB_OK | MB_ICONINFORMATION);
						return TRUE;
					}
					pThis->m_ReaderType=(CCoreEngine::CardReaderType)DlgListBox_GetItemData(hDlg,IDC_CARDREADER_READERLIST,Sel);
					LRESULT Length=DlgListBox_GetStringLength(hDlg,IDC_CARDREADER_READERLIST,Sel);
					if (Length>0) {
						LPTSTR pszName=new TCHAR[Length+1];
						DlgListBox_GetString(hDlg,IDC_CARDREADER_READERLIST,Sel,pszName);
						pThis->m_ReaderName.Set(pszName);
						delete [] pszName;
					} else {
						pThis->m_ReaderName.Clear();
					}
				} else {
					pThis->m_ReaderType=CCoreEngine::CARDREADER_NONE;
					pThis->m_ReaderName.Clear();
				}
			}
		case IDCANCEL:
			::EndDialog(hDlg,LOWORD(wParam));
			return TRUE;
		}
		return TRUE;
	}
	return FALSE;
}
